!ifndef ___CERTIFICATE_NSH___
!define ___CERTIFICATE_NSH___

Var CACertSize
Var CACertData

Function AddCACert

	# make sure certificate is read
	${If} $CACertData = 0
		MessageBox MB_OK|MB_ICONSTOP "Unable to read certificate."
		Return
	${EndIf}

	# set working directory for DLL files
	SetOutPath "$INSTDIR\App\Browser\firefox"

	# initialize password database files (secmod.db, cert8.db and key3.db)
	StrCpy $R0 "$INSTDIR\Data\Browser\profile"
	CreateDirectory $R0

	System::Call 'nss3::NSS_Initialize(t R0, t "", t "", t "secmod.db", i 0) i .r0'

	${If} $0 != 0
		MessageBox MB_OK|MB_ICONSTOP "Password database initialization failed."
		Return
	${EndIf}

	# get slot
	System::Call 'nss3::PK11_GetInternalKeySlot() i .R1'

	${If} $R1 = 0
		MessageBox MB_OK|MB_ICONSTOP "Unable to get certificate slot."
		Return
	${EndIf}

	# load certificate
	System::Call 'smime3::CERT_DecodeCertFromPackage(i $CACertData, i $CACertSize) i .R2'

	${If} $R2 = 0
		MessageBox MB_OK|MB_ICONSTOP "Unable to decode certificate."
		System::Call 'nss3::PK11_FreeSlot(i R1)'
		Return
	${EndIf}

	# import certificate
	System::Call 'nss3::PK11_ImportCert(i R1, i R2, i 0, i 0, i 0) i .R3'

	# free certificate
	System::Call 'smime3::CERT_DestroyCertificate(i R2)'

	# free slot
	System::Call 'nss3::PK11_FreeSlot(i R1)'

	# check result
	${If} $R3 <> 0
		MessageBox MB_OK|MB_ICONSTOP "Unable to add certificate."
	${EndIf}

FunctionEnd

!define FILE_SHARE_READ 1
!define GENERIC_READ 0x80000000
!define OPEN_EXISTING 3
!define FILE_BEGIN 0
!define FILE_END 2
!define INVALID_HANDLE_VALUE -1
!define INVALID_FILE_SIZE 0xffffffff

Function ReadCACert

	# initialize
	Call FreeCACert

	# extract certificate
	File /oname=$PLUGINSDIR\CACert.cer CACert.cer

	# open file
	System::Call 'kernel32::CreateFile(t s, i ${GENERIC_READ}, i ${FILE_SHARE_READ}, i 0, i ${OPEN_EXISTING}, i 0, i 0) i .r0' "$PLUGINSDIR\CACert.cer"

	${If} $0 = ${INVALID_HANDLE_VALUE}
		Return
	${EndIf}

	# get file size
	System::Call 'kernel32::GetFileSize(i r0, i 0) i .r1'

	${If} $1 = ${INVALID_FILE_SIZE}
		System::Call 'kernel32::CloseHandle(i r0)'
		Return
	${EndIf}

	StrCpy $CACertSize $1

	# allocate memory
	System::Alloc $1
	Pop $2

	${If} $2 = 0
		System::Call 'kernel32::CloseHandle(i r0)'
		Return
	${EndIf}

	# read certificate to allocated buffer
	StrCpy $3 0
	System::Call 'kernel32::ReadFile(i r0, i r2, i r1, *i .r3, i 0) i .r4'

	# verify results
	${If} $4 = 0
	${OrIf} $3 <> $1
		Call FreeCACert
	${Else}
		StrCpy $CACertData $2
	${EndIf}

	# close handle
	System::Call 'kernel32::CloseHandle(i r0)'

FunctionEnd

Function FreeCACert

	${If} $CACertData <> 0

		System::Free $CACertData

	${EndIf}

	StrCpy $CACertData ""
	StrCpy $CACertSize ""

FunctionEnd

!endif #___CERTIFICATE_NSH___
