Function AcquireAccountData

	${Do} ;Try to download the files up to five times.
		${Download} "${CHECK_URL}" "$PLUGINSDIR\xbtype.dat"
		IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
		${If} ${FileExists} "$PLUGINSDIR\xbtype.dat"
			DetailPrint "Account Checking Data Received"
			${ExitDo}
		${EndIf}
	${LoopUntil} $RETRYCOUNTER == $RETRYMAX
	
	${If} $RETRYCOUNTER < $RETRYMAX ;If we got it in time, then return.
		Return
	${Else} 						; If we didn't get it...
		${UpdateBanner} "Failed to Acquire Account Data" 
		DetailPrint "Failed To Acquire Account Data"
		MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Error Connecting to Validation Server.$\rPlease Check Your Connection/Proxy Settings."
		Call CheckForFirewall
		Call RegisterAbort
	${EndIf}

FunctionEnd

Function CheckXB2Account

	Push $0

	StrCpy $0 ""

	${If} $REG == ""
		Goto done
	${EndIf}

	${StrFilter} $REG 1 "-" "" $0 ;Filter out dashes and letters

	${If} $REG == $0 ;If there were only numbers
		StrCpy $0 "xb2"
	${Else} ;There were numbers and letters
		StrCpy $0 "xb1"
	${EndIf}

done:

	Exch $0

FunctionEnd

Function CheckAccountType

	Call AcquireAccountData
	FileOpen $8 "$PLUGINSDIR\xbtype.dat" r ;Open the file
	FileRead $8 $2 ;Read the first line
	${If} $2 == "xERROR: No such TransID (15)"
		StrCpy $ACCOUNT_TYPE "NOSUCH"
		FileClose $8
		Return
	${EndIf}
	FileRead $8 $2 13 ;Read the second line into the register
	${If} $2 == "<!-- TYPE:VPN"
		StrCpy $ACCOUNT_TYPE "XBPRO"
	${ElseIf} $2 == "<!-- TYPE:SSH"
		StrCpy $ACCOUNT_TYPE "XBPLUS"
	${ElseIf} $2 == "<!-- TYPE:ACCESS"
		StrCpy $ACCOUNT_TYPE "XBACCESS"
	${ElseIf} $2 == "<!-- TYPE:DEPOSIT"
		StrCpy $ACCOUNT_TYPE "XBDEPOSIT"
	${ElseIf} $2 == "<!-- TYPE:ERR"
		StrCpy $ACCOUNT_TYPE "ERROR"
	${Else}
		StrCpy $ACCOUNT_TYPE "CONNECTIONERROR"
	${EndIf}
	FileClose $8
	Delete "$INSTDIR\xbtype.dat"

FunctionEnd

Function DownloadSSHFilesXB1
	${UpdateBanner} "Downloading SSH Files..."
	${Download} "${USERFILE_URL}" "$INSTDIR\Data\XeroBank\xerobank.usr"
	${Download} "${KEYFILE_URL}" "$INSTDIR\Data\XeroBank\xerobank.key"
	DetailPrint "Attempting to Download Keyfiles...$RETRYCOUNTER"
FunctionEnd

Function DownloadVPNFilesXB1
	${UpdateBanner} "Downloading VPN Files..."
	${Download} "${VPNFILE_URL}" "$PLUGINSDIR\VPN_credentials.zip"
	DetailPrint "Attemping to Download Keyfiles...$RETRYCOUNTER"
FunctionEnd

Function AcquireCredentials
	${If} $RETRYCOUNTER > $RETRYMAX ;If we didn't succeed...
		${UpdateBanner} "Failed to Download Key Files"
		DetailPrint "Failed to Download Keyfiles"
		MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Error Downloading Configuration Files. Check Connection Settings and Try Again."	; Check to see if files downloaded
		Call CheckForFirewall
		Call RegisterAbort
	${EndIf}
	${If} $ACCOUNT_TYPE == "XBPLUS" 
		Call GetSSHCredentials
		Call GetMailCredentials
	${ElseIf} $ACCOUNT_TYPE == "XBPRO"
		Call GetVPNCredentials
		Call GetMailCredentials
	${ElseIf} $ACCOUNT_TYPE == "XBACCESS"	
		Call GetSSHCredentials
		Call GetVPNCredentials
		Call GetMailCredentials
	${EndIf}
	
	${If} $RETRYCOUNTER < $RETRYMAX ;If we got it, then return.
		Return
	${Else} 				; If we didn't get it...
		${UpdateBanner} "Failed to Download Credentials"
		DetailPrint "Failed to Download Credentials"
		MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Error Downloading Configuration Files. Check Connection Settings and Try Again."	; Check to see if files downloaded
		Call CheckForFirewall
		Call RegisterAbort
	${EndIf}
FunctionEnd

Function GetVPNCredentials
		${Do} ;Try to download the files up to five times.
			Call DownloadVPNFilesXB1
			IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
			${If} ${FileExists} "$PLUGINSDIR\VPN_credentials.zip"		; Check to see if files downloaded
				FileOpen $8 "$PLUGINSDIR\VPN_credentials.zip" r ;Open the key file
				FileRead $8 $2 ;Read the first line
				FileClose $8
				${If} $2 == "xERROR: Invalid account id. (-1)" ;Let's see if there is a backend error.
				${OrIf} $2 == "xERROR: Packet creation failed (3)"
				${OrIf} $2 == "xERROR: Packet fetch failed (4)"
					${UpdateBanner} "Error Acquiring Credentials for $REG. Retrying..."
					Sleep 1000
					IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
					Call AcquireCredentials
				${EndIf}
				${UpdateBanner} "VPN Credentials Downloaded"
				DetailPrint "VPN Credentials Download Complete"
				${UpdateBanner} "Unpacking VPN Credentials"
				Call ExtractVPNCredentials
				${ExitDo}
			${EndIf}
		${LoopUntil} $RETRYCOUNTER == $RETRYMAX
		;Unpack the Credentials files. How are we going to deal with VPN credentials upgrades?
FunctionEnd

Function ExtractVPNCredentials
	ClearErrors
	ZipDLL::extractall "$PLUGINSDIR\VPN_credentials.zip" "$INSTDIR\Data\config\"
	${If} ${Errors}
		MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Error extracting VPN credentials. Please retry."
		Abort
	${EndIf}
FunctionEnd

Function GetSSHCredentials
		${Do} ;Try to download the files up to five times.
			Call DownloadSSHFilesXB1
			IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
			${If} ${FileExists} "$INSTDIR\Data\XeroBank\xerobank.key"		; Check to see if files downloaded
			${AndIf} ${FileExists} "$INSTDIR\Data\XeroBank\xerobank.usr"	; Check to see if files downloaded
				FileOpen $8 "$INSTDIR\Data\XeroBank\xerobank.key" r ;Open the key file
				FileRead $8 $2 ;Read the first line
				FileClose $8
				${If} $2 == "xERROR: Invalid account id. (-1)" ;Let's see if there is a backend error.
					${UpdateBanner} "Error Acquiring Credentials for $REG. Retrying..."
					Sleep 1000
					IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
					Call AcquireCredentials
				${EndIf}
				FileOpen $8 "$INSTDIR\Data\XeroBank\xerobank.usr" r ;Open the usr file
				FileRead $8 $2 ;Read the first line
				FileClose $8
				${If} $2 == "xERROR: Invalid account id. (-1)" ;Let's see if there is a backend error.
					${UpdateBanner} "Error Acquiring Credentials for $REG. Retryiing..."
					Sleep 1000
					IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
					Call AcquireCredentials
				${EndIf}
				${UpdateBanner} "Key Files Downloaded"
				DetailPrint "Configuration Download Complete"
				${ExitDo}
			${EndIf}
		${LoopUntil} $RETRYCOUNTER == $RETRYMAX
FunctionEnd

Function AcquireXB2AccountData

	CreateDirectory $INSTDIR\Data\config

	Call DownloadXB2AccountData

	FileOpen $8 "$PLUGINSDIR\VPN_credentials.zip" r ;Open the file
	FileRead $8 $2 ;Read the first line
	FileClose $8
	${If} $2 == "ERROR"
		StrCpy $8 "" ; Dialog::InputBox will not change $8 if the user hits cancel
		Dialogs::InputBox "$(^Caption)" "Your account is protected by a PIN code. Please enter your PIN code below. If your account isn't protected by a PIN code, hit Cancel and make sure you've entered your access code correctly." "OK" "Cancel" "1" 8
		StrCpy $PIN $8
		${EncodeURL} $PIN $PIN_ENCODED

		${If} $PIN == ""
			Call RegisterAbort
		${EndIf}

		Call DownloadXB2AccountData

		FileOpen $8 "$PLUGINSDIR\VPN_credentials.zip" r ;Open the file
		FileRead $8 $2 ;Read the first line
		FileClose $8
		${If} $2 == "ERROR"
			# invalid access code
			StrCpy $PIN ""
			${EncodeURL} $PIN $PIN_ENCODED
			MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Invalid access code or PIN code. Please retry."
			Call RegisterAbort
		${EndIf}

		StrCpy $PIN_EXISTS "true"
	${EndIf}

	Call ExtractVPNCredentials

FunctionEnd

Function DownloadXB2AccountData

	StrCpy $RETRYCOUNTER 0

	${Do} ;Try to download the files up to five times.
		${Download} "${XB2VPN_URL}" "$PLUGINSDIR\VPN_credentials.zip"

		${If} ${FileExists} "$PLUGINSDIR\VPN_credentials.zip"
			DetailPrint "XB2 Account Checking Data Received"
			${ExitDo}
		${EndIf}

		IntOp $RETRYCOUNTER $RETRYCOUNTER + 1
	${LoopUntil} $RETRYCOUNTER == $RETRYMAX

	${IfNot} ${FileExists} "$PLUGINSDIR\VPN_credentials.zip" ; If we didn't get it...
		${UpdateBanner} "Failed to Acquire XB2 Account Data" 
		DetailPrint "Failed To XB2 Acquire Account Data"
		MessageBox MB_OK|MB_ICONSTOP|MB_TOPMOST "Error Connecting to Validation Server.$\rPlease Check Your Connection/Proxy Settings."
		Call CheckForFirewall
		Call RegisterAbort
	${EndIf}

FunctionEnd

Function CheckPINExists

	${UpdateBanner} "Verifying PIN code..."

	StrCpy $PIN_EXISTS false

	${Download} ${PINEXIST_URL} $PLUGINSDIR\pintest

	${If} $DOWNLOAD_RESULT != "OK"

		# XXX TODO no result, what to do?
		Return

	${EndIf}

	FileOpen $0 $PLUGINSDIR\pintest r
	FileRead $0 $1
	FileClose $0

	StrCpy $1 $1 5
	${If} $1 == "ERROR"

		StrCpy $PIN_EXISTS true

	${EndIf}

FunctionEnd

Function TestPIN

	StrCpy $0 "bad pin"

	${Download} ${PINTEST_URL} $PLUGINSDIR\pintest

	${If} $DOWNLOAD_RESULT != "OK"

		# XXX TODO no result, what to do?
		Return

	${EndIf}

	FileOpen $0 $PLUGINSDIR\pintest r
	FileRead $0 $1
	FileClose $0

	StrCpy $1 $1 5
	${If} $1 != "ERROR"

		StrCpy $0 "good pin"

	${EndIf}

FunctionEnd

Function SetPIN

	StrCpy $0 "success"

	${Download} ${PINSET_URL} $PLUGINSDIR\pinset

	${If} $DOWNLOAD_RESULT != "OK"

		StrCpy $0 $DOWNLOAD_RESULT
		Return

	${EndIf}

	FileOpen $2 $PLUGINSDIR\pinset r
	FileRead $2 $1
	FileClose $2

	${If} $1 != "OK" # XXX TODO what will the API return?

		StrCpy $0 $1

	${EndIf}

FunctionEnd
