;Copyright (C) 2008 Xero Networks AG
;All rights reserved by Xero Networks AG
;This software is licensed under the latest version of the Torrify Ethical Software License Agreement
;The TESLA License can be viewed at http://www.XeroBank.com/tesla.html
;This source code can not be used to create a commercial alternative to XeroBank services.

!ifndef ___UPDATE_NSH___
!define ___UPDATE_NSH___

Var NewVersion

Function ReadNewVersion

	Push $R0
	Push $R1
	Push $R2
	Push $R3
	Push $R4

	${DebugHeader} "ReadNewVersion called"

	StrCpy $R1 ""
	StrCpy $NewVersion ""

	ClearErrors
	FileOpen $R0 "$TEMP\New ${NAME} Version.ver" r
	${IfNot} ${Errors}
		FileRead $R0 $R1
		FileClose $R0
	${EndIf}

	${DebugHeader} "   Online version is $R1"
	${DebugHeader} "   Current version is ${VER}${SUBVERSION}"

	${StrFilter} $R1 31 . "" $R2 # numbers only, e.g. "2.0.0.11"
	${StrFilter} $R1 2 "" "" $R3 # build only, e.g. "a"
	${VersionCompare} ${VER} $R2 $R4

	${DebugHeader} "   VersionCompare returned $R4"

	${If} $R4 == 2 # newer version exists
		StrCpy $NewVersion $R1
	${ElseIf} $R4 == 0 # same number
	${AndIf} $R3 S> ${SUBVERSION} # newer build
		StrCpy $NewVersion $R1
	${EndIf}

	Pop $R4
	Pop $R3
	Pop $R2
	Pop $R1
	Pop $R0

FunctionEnd

Function InstallUpdate

	${DebugHeader} "InstallUpdate: check if downloaded update is available"

	Call ReadNewVersion
	${If} $NewVersion == ""
		# no update or old update, delete all related files
		Delete "$TEMP\New ${NAME} Version.exe"
		Delete "$TEMP\New ${NAME} Version.exe.sig"
		Delete "$TEMP\New ${NAME} Version.msg"
		Delete "$TEMP\New ${NAME} Version.ver"
		# and end
		Return
	${EndIf}

	Push $R0
	Push $R1
	Push $R2
	Push $R3
	Push $R4
	Push $R5

	${If} ${FileExists} "$TEMP\New ${NAME} Version.exe"				;The update
	${AndIf} ${FileExists} "$TEMP\New ${NAME} Version.exe.sig"		;The update signature
		InitPluginsDir
		${DebugHeader} "   Both update and signature were found"
		${DebugHeader} "   Extracting GPGV and keyring to $PLUGINSDIR"
		File /oname=$PLUGINSDIR\gpgv.exe ..\Library\tools\gpgv.exe
		File /oname=$PLUGINSDIR\sig.rng ..\Library\sigs\xerobank.rng 
		${DebugHeader} "   Running Signature Check: '$PLUGINSDIR\gpgv.exe' --keyring '$PLUGINSDIR\sig.rng' '$TEMP\New ${NAME} Version.exe'"
		nsExec::Exec '"$PLUGINSDIR\gpgv.exe" --keyring "$PLUGINSDIR\sig.rng" "$TEMP\New ${NAME} Version.exe.sig"'
		Pop $0
		${DebugHeader} "   GPGV returned $0"
		${If} $0 != 0	;if bad signature
			${DebugHeader} "   Bad Signature. Deleting updates. WARNING!"
			Delete "$TEMP\New ${NAME} Version.exe"
			Delete "$TEMP\New ${NAME} Version.exe.sig"
			Delete "$TEMP\New ${NAME} Version.msg"
			Delete "$TEMP\New ${NAME} Version.ver"
			Return
		${Else}
			${DebugHeader} "   GPGV did not return a bad signature or error. SUCCESS" ;We may want a proactive verification other than Not Fail
		${EndIf}
	${Else}
		${DebugHeader} "Either one or both of the update and update signature were not found"
	${EndIf}

	${If} ${FileExists} "$TEMP\New ${NAME} Version.exe"
	${AndIf} ${FileExists} "$TEMP\New ${NAME} Version.exe.sig"
	${AndIf} ${FileExists} "$TEMP\New ${NAME} Version.ver"
	${AndIf} ${FileExists} "$TEMP\New ${NAME} Version.msg"
		${DebugHeader} "   update is available, all four files are in place"
		${DebugHeader} "   version $NewVersion"

		FileOpen $R0 "$TEMP\New ${NAME} Version.msg" r
		StrCpy $R2 ""
		${Do}
			FileRead $R0 $R3
			${If} $R3 == ""
				${ExitDo}
			${EndIf}
			StrCpy $R2 "$R2$R3"
		${Loop}
		FileClose $R0

		${If} ${Cmd} `MessageBox MB_YESNO|MB_TOPMOST|MB_ICONEXCLAMATION "${FULLNAME} v$NewVersion is now available and ready for installation. $\n$\n$R2$\n$\nWould you like to install this version now?" IDYES`
			${DebugHeader} "   user wants to update... updating!"
			File /oname=$PLUGINSDIR\updater.exe Updater.exe
			Exec '"$PLUGINSDIR\updater.exe" /install=new'
			Abort
		${EndIf}

		${DebugHeader} "   user doesn't want to update"

	${EndIf}

	Pop $R5
	Pop $R4
	Pop $R3
	Pop $R2
	Pop $R1
	Pop $R0

FunctionEnd

Function CheckForUpdate

	${DebugHeader} "CheckForUpdate: Checking for updates on the web"

	!insertmacro Download "${UPDATELOC}" "$TEMP\New ${NAME} Version.ver" ;Get the latest version file.

	Call ReadNewVersion

	${If} $NewVersion != ""
		${If} $CONNECTIONMETHOD == 0 ;If we are using Tor...
			${DebugHeader} "   User is connected via Tor, asking about update..."
			${If} ${Cmd} `MessageBox MB_YESNO|MB_TOPMOST|MB_ICONQUESTION "${FULLNAME} $NewVersion is now available. Would you like to download it?" IDYES`
				${DebugHeader} "   Online version is newer, downloading silently."
				File /oname=$PLUGINSDIR\updater.exe Updater.exe
				Exec '"$PLUGINSDIR\updater.exe" /proxy=$INETPROXY /user=$InetAuthUser /password=$InetAuthPassword'
			${Else}
				${DebugHeader} "   User rejected update!"
			${EndIf}
		${Else}
			${DebugHeader} "   Online version is newer, downloading silently."
			File /oname=$PLUGINSDIR\updater.exe Updater.exe
			Exec '"$PLUGINSDIR\updater.exe" /proxy=$INETPROXY /user=$InetAuthUser /password=$InetAuthPassword'
		${EndIf}
	${Else}
		${DebugHeader} "   Online version is older or the same, ignoring."
	${EndIf}

FunctionEnd

!endif # ___UPDATE_NSH___
