Section "Main"

	# make sure we can run
	Call CheckMutex

	# install new versions
	Call InstallUpdate

	# initialize splash screen
	!insertmacro SplashSetText "Loading ${FULLNAME}..."

	# initialize variables
	SetVars:
		StrCpy $DEBUGON 0
		StrCpy $LaunchedExecutable 0  	;ProgramExecutable has not been launched
		StrCpy $SHUTDOWNCOUNTER 0

	# parse command line parameters
	Call ParseCmdLine

	!insertmacro SplashSetText "Loading settings..."

	# load settings file (XeroBank.ini)
	Call LoadSettings

	# disable splash screen if requested
	${If} $DisableSplashScreen == "true"
		!insertmacro SplashDestroy
	${EndIf}

	# setup profile directory
	Call SetupProfileDirectory
	Call SetupWritableDirectories
	
	# tor or xerobank?
	Call DetermineService
	Pop $0

	${If} $0 == "XeroBank"
		Goto SetToXeroBank
	${ElseIf} $0 == "Tor"
		Goto SetToTor
	${EndIf}

	SetToXeroBank:
		${DebugHeader} "SetToXeroBank"
		!insertmacro SplashSetText "XeroBank Network"
		Call GetCredentials ;Get Proxy Credentials, if there are any.
		
		;Remove the MimeTypes.rdf so it gets regenerated fresh against the current known handlers.
		;More to the point, we don't want to get stuck with the Tor mimetypes
		Delete "$PROFILEDIRECTORY\mimeTypes.rdf"
				
		;First disable FlushTorCircuit
		${DisableExtension} "Flush Tor Circuit"
		;${DisableExtension} "NoScript"		
		;${EnableExtension} "Site Security Policy"
		${If} $CONNECTIONMETHOD == 1 ; SSH
			StrCpy $SOCKSPORT 8085 ;Set the socks port, which is called in setjavaproxy
			Call SetJavaProxy ;Add additional protection against java leaks against non-malicious code.
			;Now switch out the prefs.js to the XeroBank Plus settings
			${DebugHeader} "SetToXeroBank: Now loading SSH Profile"
			!insertmacro SplashSetText "Configuring SSH..."
			${Unless} ${FileExists} "$PROFILEDIRECTORY\prefs.js"
			${OrUnless} ${FileExists} "$PROFILEDIRECTORY\SSH_user.js"
				;Make sure a XeroBank preference profile exists.
				${DebugHeader} "     Restoring SSH preference files"
				!insertmacro SplashSetText "Restoring SSH profile..."
				CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\prefs_default.js" "$PROFILEDIRECTORY\prefs.js" ;If one doesn't exist, restore it from the backup.
				CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\SSH_user_default.js" "$PROFILEDIRECTORY\SSH_user.js" ;If one doesn't exist, restore it from the backup.
			${EndIf}

			CopyFiles /SILENT "$PROFILEDIRECTORY\SSH_user.js" "$PROFILEDIRECTORY\user.js" ;Copy it to be the current profile used for the client.
			StrCpy $PREFSLOADED 1

			${DebugHeader} "     SSH Profile is now loaded"			
			
			${DebugHeader} "Loading XeroBank.usr..."
			!insertmacro SplashSetText "Reading user info..."	
			;READING THE INI FILE: XeroBank Section
			ClearErrors 
			ReadINIStr $XeroBankUSERNAME "$EXEDIR\Data\XeroBank\XeroBank.usr" "XeroBank" "username" ;Read the username from XeroBank.usr into variable XeroBankUSERNAME
			${If} ${Errors}
				MessageBox MB_OK|MB_ICONINFORMATION `Error in $EXEDIR\Data\XeroBank\XeroBank.usr`
				Call UnloadGUIandClient
			${EndIf}
				
			;Setting the ConnectionClient Information
			StrCpy "$CONNECTIONCLIENT" "XeroBankPlus.exe"
			${Unless} ${FileExists} $EXEDIR\App\XeroBank\$CONNECTIONCLIENT
				MessageBox MB_OK "$EXEDIR\App\XeroBank\$CONNECTIONCLIENT Not Found!$\n$\nPlease download a fresh copy of ${FULLNAME} from your client account at ${WEBSITE}"
				Call UnloadGUIandClient
			${EndIf}

			StrCpy "$CONNECTIONCLIENTNAME" ${SERVICENAME}
			${If} $HTTPPROXYADDRESS != "";
			${OrIf} $HTTPSPROXYADDRESS != "";
				${DebugHeader} "Creating PuTTY Session File"
				!insertmacro SplashSetText "Configuring local proxy..."
				Call CreatePuTTYSettings
				StrCpy "$CONNECTIONSTRING" `"$EXEDIR\App\XeroBank\$CONNECTIONCLIENT" XeroBank -v -v -batch` ;Use the Putty Profile named 'XeroBank'
			${Else}
				StrCpy "$CONNECTIONSTRING" `"$EXEDIR\App\XeroBank\$CONNECTIONCLIENT" $XeroBankUSERNAME@plus.xerobank.com -i "$EXEDIR\Data\XeroBank\XeroBank.key" -P 443 -L 8080:httpproxy.xerobank.com:8080 -L $SOCKSPORT:socksproxy.xerobank.com:$SOCKSPORT -v -v -batch`
			${EndIf}
			${DebugHeader} "     Connection String = $CONNECTIONSTRING"
			StrCpy "$GotConnection" 1 ;Assume we have a connection!
			Goto EndINI
		${EndIf}
		
		${If} $CONNECTIONMETHOD == 2 ;VPN
			${If} ${IsProcessRunning} "${VPNCLIENT}" ;If xb VPN is running...
			${OrIf} ${IsProcessRunning} ${LEGACYVPN}
			${OrIf} ${IsProcessRunning} ${LEGACYVPNGUI}
				;Now switch out the prefs.js to use the XeroBank Pro settings
				${DebugHeader} "SetToXeroBank: Now loading VPN profile"
				!insertmacro SplashSetText "Loading VPN profile..."
				${Unless} ${FileExists} "$PROFILEDIRECTORY\prefs.js"
				${OrUnless} ${FileExists} "$PROFILEDIRECTORY\VPN_user.js"
					;Make sure a XeroBank preference profile exists.
					${DebugHeader} "     Restoring VPN preference files"
					!insertmacro SplashSetText "Restoring VPN profile..."
					CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\prefs_default.js" "$PROFILEDIRECTORY\prefs.js" ;If one doesn't exist, restore it from the backup.
					CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\VPN_user_default.js" "$PROFILEDIRECTORY\VPN_user.js" ;If one doesn't exist, restore it from the backup.
				${EndIf}

				CopyFiles /SILENT "$PROFILEDIRECTORY\VPN_user.js" "$PROFILEDIRECTORY\user.js" ;Copy it to be the current profile used for the client.
				StrCpy $PREFSLOADED 1

				${DebugHeader} "     VPN_prefs.js is now loaded"
				
				;Setting the ConnectionClient Information
				StrCpy "$CONNECTIONCLIENT" "${VPNCLIENT}"
				StrCpy "$CONNECTIONCLIENTNAME" ${SERVICENAME}
				StrCpy "$CONNECTIONSTRING" ``
				StrCpy "$GotConnection" 1 ;Assume we have a connection!
				StrCpy "$SUBMISSIVEMODE" "true" ;We don't want to bother with connections.
				Goto EndINI			
			${Else} ;If xB VPN Isn't running, but CONNECTIONMETHOD is set to 2...
				MessageBox MB_OK "Please start xB VPN before running ${FULLNAME}. If this is the first time you've run xB VPN, please reboot."
				Call UnloadGUI
			${EndIf}
		${EndIf}
				
	SetToTor:
		${DebugHeader} "SetToTor"
		!insertmacro SplashSetText "Tor Network"
		Call SetJavaProxy ;Help protect against non-malicious java applet leaks.
		Call GetCredentials ;Get proxy credentials, if there are any.
		Call StopPluginsLoading
		
		StrCpy $CONNECTIONMETHOD 0
		;First enable FlushTorCircuit
		${DebugHeader} "SetToTor | Getting ready to load FTC..."
		;${DisableExtension} "Site Security Policy"
		${EnableExtension} "Flush Tor Circuit"
		;${EnableExtension} "NoScript"
		;Now switch out the prefs
		${Unless} ${FileExists} "$PROFILEDIRECTORY\prefs.js"
		${OrUnless} ${FileExists} "$PROFILEDIRECTORY\TOR_user.js"
			;Make sure a tor preference profile exists.
				${DebugHeader} "     Restoring Tor Preference files..."
				!insertmacro SplashSetText "Restoring Tor profile..."
			;Restore from backup
			CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\prefs_default.js" "$PROFILEDIRECTORY\prefs.js"
			CopyFiles /SILENT "$EXEDIR\Data\Browser\backups\TOR_user_default.js" "$PROFILEDIRECTORY\TOR_user.js"
		${EndIf}
		;Copy it to be the current profile used for the client.
		CopyFiles /SILENT "$PROFILEDIRECTORY\TOR_user.js" "$PROFILEDIRECTORY\user.js"
		StrCpy $PREFSLOADED 1
		${DebugHeader} "     Tor Preferences Loaded"
		!insertmacro SplashSetText "Tor preferences loaded..."
		StrCpy "$CONNECTIONCLIENT" "tor.exe"
		StrCpy "$CONNECTIONCLIENTNAME" "Tor"
		StrCpy "$STANDARDSTRING" `SOCKSPort $SOCKSPORT ControlPort $CONTROLPORT AllowUnverifiedNodes middle,rendezvous SafeLogging 1 ClientOnly 1 DataDirectory tor\data LongLivedPorts "21, 22, 443, 706, 1863, 5050, 5190, 5222, 5223, 6667, 8300, 8888" AvoidDiskWrites 1 CircuitBuildTimeout 8`
		${If} $DEBUGON > 0 ;If we are in debug mode, create a debug log for tor as well!
			StrCpy $STANDARDSTRING `$STANDARDSTRING log "debug file $DESKTOP\${NAME}_Tor.log"`
		${EndIf}
		;Now check the proxy data and see what we need to do
			;http proxy data...
		
		StrCpy $CONNECTIONSTRING `"$TORDIRECTORY\tor.exe" $STANDARDSTRING $PROXYSTRING1 $AUTHSTRING1 $PROXYSTRING2 $AUTHSTRING2 $TORCOMMANDS`
		StrCpy $GotConnection 0 ;Assume there is no connection yet.
		
		${DebugHeader} "  	ConnectionString = $TORDIRECTORY\tor.exe |$STANDARDSTRING |$PROXYSTRING1 |$AUTHSTRING1 |$PROXYSTRING2 |$AUTHSTRING2 |$TORCOMMANDS"
		Goto EndINI	 
	 
	EndINI:
		${DebugHeader} "EndINI"
		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundClientExecutable NoClientExecutable

	NoClientExecutable:					; Program executable was not found where expected
		${DebugHeader} "NoClientExecutable"
		MessageBox MB_OK|MB_ICONEXCLAMATION `$PROGRAMEXECUTABLE was not found.  Please check your configuration. $PROGRAMEXECUTABLE should be located in $\n$PROGRAMDIRECTORY$\n${FULLNAME} will now exit.`
		Call UnloadGUI
		
	FoundClientExecutable: 				;Check and see if client is already running
		${DebugHeader} "FoundClientExecutable"
		;${If} ${IsProcessRunning} $PROGRAMEXECUTABLE
			; if client is already running, kill it?
		;	${If} ${Cmd} `MessageBox MB_YESNO|MB_ICONQUESTION|MB_TOPMOST "${APP} is already running. Close ${APP} and continue starting ${FULLNAME}?" IDNO`
		;		Call UnloadGUI
		;	${EndIf}
		;	${DebugHeader} "KeepLoadingBrowser"
		;	KillProcDLL::KillProc "$PROGRAMEXECUTABLE" 
		;	Sleep 1000
		;${EndIf}
		Goto ProcessProfile

	ProcessProfile: ;Check for an existing profile
		${DebugHeader} "ProcessProfile"
		IfFileExists "$PROFILEDIRECTORY\prefs.js" DisplaySplash
		${DebugHeader} "ProcessProfile: $PROFILEDIRECTORY\prefs.js not found"
			;No profile was found
			!insertmacro SplashSetText "No profile found"
			StrCmp $ISDEFAULTDIRECTORY "true" CopyDefaultProfile
			${DebugHeader} "ProcessProfile: ISDEFAULTDIRECTORY = $ISDEFAULTDIRECTORY"
			MessageBox MB_YESNO|MB_ICONQUESTION|MB_TOPMOST `No profile was found. Create a new profile here? $\n$PROFILEDIRECTORY\` IDYES DisplaySplash
			MessageBox MB_OK|MB_ICONINFORMATION|MB_TOPMOST `${NAME} can not run without a profile directory.  Please check your configuration.`
			Call UnloadGUI
	
	CopyDefaultProfile:
		${DebugHeader} "CopyDefaultProfile"
		CreateDirectory "$EXEDIR\Data"
		CreateDirectory "$EXEDIR\Data\Browser\plugins"
		CreateDirectory "$EXEDIR\Data\Browser\profile"
		CopyFiles /SILENT $EXEDIR\App\Browser\DefaultData\Browser\plugins\*.* $EXEDIR\Data\Browser\plugins
		CopyFiles /SILENT $EXEDIR\App\Browser\DefaultData\Browser\profile\*.* $EXEDIR\Data\Browser\profile
		GoTo DisplaySplash
		
	DisplaySplash:
		${DebugHeader} "DisplaySplash"
		${If} $DISABLEWARNING != "true" ;We are disabling the warning. Doesn't really servce much warning, just an impedement.
			;${DebugHeader} "      DisplaySplash: Now showing the warning"
			;MessageBox MB_YESNO|MB_TOPMOST "${NAME} creates an anonymous encrypted connection, but once data leaves the$\ranonymous network it is no longer encrypted but is still anonymized.$\rDo you want to continue?" IDNO UnloadGUIandClient
		${EndIf}

	Call SetEnvironmentVariables

	LaunchConnection:
		${DebugHeader} "LaunchConnection. CONNECTIONMETHOD = $CONNECTIONMETHOD; ClientRestartCounter = $CLIENTRESTARTCOUNTER"
		!insertmacro SplashSetText "Opening network connection..."
		Call CheckForBadclient ;Make sure no client is running before Browser has called it.
		
		${If} $SUBMISSIVEMODE == "true" ;If we aren't waiting for tor, bust out of here and load the GUI.
			${DebugHeader} "   Submissive mode active."
			StrCpy $GOTCONNECTION "1" ;Fake that we have a connection.
			Goto LaunchGUIClient
		${EndIf}
		
		Call CONNECTIONHANDLER
		${If} $R0 == "1" ; If the connection client IS running 
			Goto CheckForCircuit
		${EndIf}
		${If} $CLIENTRESTARTCOUNTER > 25 ;If we have restarted tor more than 5 times, there must be a real problem. Quit the program and notify the user.
			${DebugHeader} "ClientRestartCounter has reached the limit."
			MessageBox MB_OK|MB_TOPMOST "${NAME} has restarted $CONNECTIONCLIENTNAME more than 5 times. This suggests your $CONNECTIONCLIENTNAME configuration$\nsettings for ${NAME} are incorrect. Please recheck your settings."
			Call CheckFirewalls			
			ExecWait '"$EXEDIR\App\xBconfig.exe"'
			Call UnloadGUIandClient
		${EndIf}
		Call LaunchConnectionClient ;Launch the connection client
		Goto RestartTimer

	
	RestartTimer:
		${DebugHeader} "RestartTimer"

		# check for bad key
		Call PuttyConnectionChecker
		Call CheckForBadclient
		${time::GetLocalTime} $LAUNCHTIME
		Goto LaunchConnection ;Rinse and repeat

	CheckForCircuit: ;Checking to see if there is a Tor circuit currently active. If not, we'll wait before launching the browser.
		${DebugHeader} "CheckForCircuit"
		Call Throbber
		Call CheckForBadclient ;Make sure nobody else has launched firefox or thunderbird without our permission.
		Call CONNECTIONHANDLER ;Check and see that the connection client program (tor or plink) is running
		${If} $R0 == "0";If it isn't running...
			Goto LaunchConnection ;Then Restart it
		${EndIf}
		ClearErrors
		Call CheckTheTime
		IntCmp $CIRCUITTIMER $TIMERMAX CONNECTIONTIMEOUT "" CONNECTIONTIMEOUT ;If the circuittimer equals X time, then go to CONNECTIONTIMEOUT, else follow the lines below
		${If} $WAITFORTORCIRCUIT == "false" ;If we aren't waiting for a circuit...
			${DebugHeader} "WaitForTorCircuit = $WAITFORTORCIRCUIT; Skipping CheckForCircuit"
			!insertmacro SplashSetText "CONNECTED"
			Sleep 1000
			Goto LaunchGUIClient
		${EndIf}
		${If} $GotConnection == "1" ;If we have a connection, launch the client.
			!insertmacro SplashSetText "CONNECTED"
			Sleep 1000
			Goto LaunchGUIClient
		${EndIf}	
		TorCircuitStatus::CountCircuits $CONTROLPORT
		Pop $NUMCIRCUITS ;Get the error code off the stack
		Goto FlagCheck
		
	FlagCheck: ;Check and see what the error code is returned from TorCircuitStatus
		${DebugHeader} "FlagCheck - $NUMCIRCUITS"
		${If} $NUMCIRCUITS == "error"
			Goto CheckForCircuit
		${ElseIf} $NUMCIRCUITS = 0
			Goto LaunchConnection
		${ElseIf} $NUMCIRCUITS < 250
			!insertmacro SplashSetText "CONNECTED"
			Sleep 1000
			Goto LaunchGUIClient
		${Else}
			Call ConnectionHandler
			Goto CheckForCircuit
		${EndIf}
				
	CONNECTIONTIMEOUT: ;This procedure will tell Tor users if it is taking too long to get a circuit.
		${DebugHeader} "ConnectionTimeout: $CIRCUITTIMER / $TIMERMAX seconds"
		${If} $CONNECTIONMETHOD == 0
			!insertmacro SplashSetText "Connection timeout"
		${EndIf}
		Call CheckForBadclient
		!insertmacro SplashDestroy ;kill the splash screen if it is still here somehow
		messagebox::show MB_DEFBUTTON4|MB_TOPMOST `"Tor Timeout" \
			"0,103" \	
			"It appears that the $CONNECTIONCLIENTNAME process has been unable to establish a connection after $TIMERMAX seconds.$\nWould you like to keep waiting for a circuit, restart the $CONNECTIONCLIENTNAME process, or quit ${NAME}? $\n $\nConnections to the Tor netork can be very slow at times, taking up to 10 minutes to establish a circuit. $\nIf you are behind a firewall or proxy, and trying to use the Tor network, you must run Tconfig and provide$\nyour network settings. $\n$\nIf you are unsatisfied with the speed or connectivity of the Tor network, consider subscribing to XeroBank services$\nfor no-waiting high-speed access. You can subscribe by visiting ${WEBSITE}" \
			"Wait Another $TIMERMAX Seconds" "${NAME} Configuration" "Quit ${NAME}" \`
			Pop $0
			${If} $0 == 1
				Goto RestartTimer
			${ElseIf} $0 == 2
				ExecWait "$EXEDIR\App\xBconfig.exe"
				Goto SetVars
			${Else}
				Call UnloadGUIandClient
			${EndIf}
			
	LaunchGUIClient: ;Launching client such as the browser or email program
		${DebugHeader} "LaunchGUIClient: ShutDownCounter = $SHUTDOWNCOUNTER"
		${If} $SHUTDOWNCOUNTER > 0
			;Do nothing. Value may not be set.
		${Else}
			Call CheckForBadclient
			Delete $PROFILEDIRECTORY\parent.lock
			${If} ${FileExists} $PROFILEDIRECTORY\parent.lock ;find out if client is running
				${If} $LaunchedExecutable == 1  ;If client is already running (we killed tor and looped back here), then don't try to run client again. else, keep going.
					${DebugHeader} "   Parent Lock Found: Client Already Launched"
					Goto StillRunning
				${Else}
					${DebugHeader} "   Parent Lock Found: Client NOT Launched. WARNING!"
				${EndIf}
			${Else}
				${DebugHeader} "   Parent Lock Not Found: GUI client not running. Proceeding normally."
			${EndIf}

			StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" "${STARTURL}" -no-remote -turbo -profile "$PROFILEDIRECTORY"`
			StrCpy $LaunchedExecutable 1 ;This is to know if we actually launched the client process. This will be referenced in CheckForCircuit so someone doesn't start client while we are looking for a circuit.
			Call RemoveEvil ;Remove things that may harm or track the GUI client
			!insertmacro SplashSetText "Loading browser..."
			Exec $EXECSTRING ; Launch client

			;Because from a USB stick it might take a while to load ff, we want to make sure it doesn't go to stillrunning and quit.
			${DoUntil} ${FileExists} $PROFILEDIRECTORY\parent.lock
				${DebugHeader} "Passoff"
				Sleep 1000
			${Loop}

			!insertmacro SplashDestroy ;Stop the splash screen if it is still up
		${EndIf}
	
	StillRunning: ;This process is necessary since client respawns its own process and falsely appears to terminate. This is why there is a lag between the exit of the browser and Browser.exe termination
		${DebugHeaderLogOnly} "StillRunning"
		Call CheckUpdate
		
		Call CONNECTIONHANDLER
		${If} $R0 == "0" ;If it isn't running
			${DebugHeaderLogOnly} "Called ConnectionHandler, found connection to not be running. R0 = $R0"
			Goto LaunchConnection
		${EndIf}
		
		${If} $GOTCONNECTION == 0 ;This set of instructions is for those who have skipped from waiting. If we haven't yet got a connection..
	  	 ${AndIf} $WAITFORTORCIRCUIT == "false" ;And if we aren't waiting for a circuit... we need to let the user know when we actually have one.
			TorCircuitStatus::CountCircuits $CONTROLPORT
			Pop $NUMCIRCUITS ;Get the error code off the stack
			${If} $NUMCIRCUITS > 0 ;So if the return code is greater than 0
			 ${AndIf} $NUMCIRCUITS < 10 ;And less than ten (I can't imagine more than 10 circuits at once) We must have a circuit
				StrCpy $GOTCONNECTION 1 ;Let everyone know we got a connection.
				MessageBox MB_OK|MB_TOPMOST "${NAME} has established an anonymized connection. You may now surf securely." ;also let the user know.
			${EndIf}
		${EndIf}
		
		Delete $PROFILEDIRECTORY\parent.lock ;try to delete parent.lock. If we can, then the GUI must have exited.
		Sleep 2000 ; If client is running, check back in 2 seconds, otherwise kill the connection client
		
		${If} ${FileExists} $PROFILEDIRECTORY\parent.lock ;If we couldn't delete it, GUI must still be running
			Goto StillRunning
		${Else}
			${DebugHeader} "   parent.lock was NOT found. Checking for alternative reasons it isn't there..."
			${If} ${IsProcessRunning} "Updater.exe" ;if the updater is running wait for it to finish.
				${DebugHeader} "   Updater was found to be running. Reverting back..."
				Goto StillRunning
			${Else} ;Check one last time for parent lock
				Sleep 1000
				${If} ${FileExists} $PROFILEDIRECTORY\parent.lock ;If we couldn't delete it, GUI must still be running
					${DebugHeader} "   parent.lock was found"
					Goto StillRunning
				${ElseIf} $SHUTDOWNCOUNTER < 10 
					IntOp $SHUTDOWNCOUNTER $SHUTDOWNCOUNTER + 1
					${DebugHeader} "   shutdown counter = $SHUTDOWNCOUNTER"
					Goto StillRunning
				${ElseIf} $SHUTDOWNCOUNTER >= 10
					${DebugHeader} "   parent.lock was still not found. Assuming normal shutdown"
					${DebugHeader} "   shutdown counter = $SHUTDOWNCOUNTER"
					Call UnloadGUI
				${Else}
					Goto StillRunning
				${EndIf}
			${EndIf}
		${EndIf}
	
	;NoTimeout:
	;	${DebugHeader} "NoTimeout"
	;	IntOp $TIMERMAX 0 + 9999999999
	;	Goto CheckForCircuit

SectionEnd

;Make sure xB Browser isn't already running
Function CheckMutex

	System::Call 'kernel32::CreateMutexA(i 0, i 0, t "${NAME}Mutex") i .r1 ?e'
	Pop $R5
	${If} $R5 != 0
		Quit ;Shutdown if we are already running xB Browser.
	${EndIf}

FunctionEnd
	
;Check for firewalls. This could be cowtrops if the user doesn't understand the risk.
Function CheckFirewalls

	${If} ${IsProcessRunning} "ccapp.exe"
	${OrIf} ${IsProcessRunning} "zapro.exe"
	${OrIf} ${IsProcessRunning} "armor2net.exe"
	${OrIf} ${IsProcessRunning} "amon.exe"
	${OrIf} ${IsProcessRunning} "MpfService.exe"
	${OrIf} ${IsProcessRunning} "zonealarm.exe"
	${OrIf} ${IsProcessRunning} "outpost.exe"
	${OrIf} ${IsProcessRunning} "firewall.exe"
	${OrIf} ${IsProcessRunning} "atguard.exe"
	${OrIf} ${IsProcessRunning} "tpfw.exe"
	${OrIf} ${IsProcessRunning} "kpf4ss.exe"
	${OrIf} ${IsProcessRunning} "nprotect.exe"
	${OrIf} ${IsProcessRunning} "kpf4gui.exe"
		${If} ${Cmd} `MessageBox MB_YESNO|MB_ICONSTOP|MB_TOPMOST "${FULLNAME} has determined you may be running a firewall. Due to$\nthe nature of this program, it may cause stability problems if the$\nfirewall is not properly configured for XeroBank.$\nDo you want to continue?" IDNO`
			Quit
		${EndIf}
	${EndIf}

FunctionEnd

;Parse command line arguments
Function ParseCmdLine

	${GetParameters} $1
	${If} $1 == "/debug"
		StrCpy $DEBUGON 1
	${ElseIf} $1 == "/log"
		StrCpy $DEBUGON 2
		Call OpenDebugLog
	${ElseIf} $1 == "/DEADBEEF"
		Call EasterEgg
	${EndIf}

FunctionEnd

##########################
# Settings File Functions
#

Function LoadSettings

	Call LoadDefaultSettings

	${DebugHeader} "FindINI"
	${If} ${FileExists} "$EXEDIR\XeroBank.ini" ; If there is an INI file, it has to be in the EXEDIR with the main program.
		${DebugHeader} "INI File Found"
		StrCpy "$INIPATH" "$EXEDIR\"
		Call LoadINISettings
	${EndIf}

	Call LogINISettings

FunctionEnd

!macro ReadINISettings VAR SECTION SETTING PREFIX
	ClearErrors
	ReadINIStr ${Var} "$INIPATH\XeroBank.ini" "${SECTION}" "${SETTING}"
	${If} ${Errors}
		MessageBox MB_OK|MB_TOPMOST|MB_ICONINFORMATION `INI Error in ${SETTING}`
	${Else}
		StrCpy ${VAR} "${PREFIX}${VAR}"
	${EndIf}
!macroend

!macro ReadXeroBankSettings VAR SETTING
	!insertmacro ReadINISettings ${VAR} XeroBank "${SETTING}" ""
!macroend
!define ReadXeroBankSettings `!insertmacro ReadXeroBankSettings`

!macro ReadXeroBankPathSettings VAR SETTING
	!insertmacro ReadINISettings ${VAR} XeroBank "${SETTING}" "$EXEDIR\"
!macroend
!define ReadXeroBankPathSettings `!insertmacro ReadXeroBankPathSettings`

!macro ReadProgramSettings VAR SETTING
	!insertmacro ReadINISettings ${VAR} ${NAME} "${SETTING}" ""
!macroend
!define ReadProgramSettings `!insertmacro ReadProgramSettings`

!macro ReadProgramPathSettings VAR SETTING
	!insertmacro ReadINISettings ${VAR} ${NAME} "${SETTING}" "$EXEDIR\"
!macroend
!define ReadProgramPathSettings `!insertmacro ReadProgramPathSettings`

Function LoadINISettings

	${DebugHeader} "ReadINI"
	!insertmacro SplashSetText "Reading settings..."
	;XeroBank section
	${ReadXeroBankPathSettings} $TORDIRECTORY TorDirectory
	${ReadXeroBankPathSettings} $OutPath OutPath
	${ReadXeroBankSettings} $TIMERMAX TimerMax
	${ReadXeroBankSettings} $CONNECTIONMETHOD CONNECTIONMETHOD
	${ReadXeroBankSettings} $WaitForTorCircuit WaitForTorCircuit
	${ReadXeroBankSettings} $SUBMISSIVEMODE SubmissiveMode
	${ReadXeroBankSettings} $SOCKSPORT SOCKSPort
	${ReadXeroBankSettings} $CONTROLPORT ControlPort
	${ReadXeroBankSettings} $DeleteDirData DeleteDirData
	${ReadXeroBankSettings} $HTTPPROXYADDRESS HTTPProxyAddress
	${ReadXeroBankSettings} $HTTPPROXYPORT HTTPProxyPort
	${ReadXeroBankSettings} $HTTPPROXYAUTH HTTPProxyAuth
	${ReadXeroBankSettings} $HTTPSPROXYADDRESS HTTPSProxyAddress
	${ReadXeroBankSettings} $HTTPSPROXYPORT HTTPSProxyPort
	${ReadXeroBankSettings} $HTTPSPROXYAUTH HTTPSProxyAuth
	${ReadXeroBankSettings} $DEBUGON DebugOn
	${ReadXeroBankSettings} $TORCOMMANDS TorCommands

	;Program Section ex: Browser, Mail, etc.
	${ReadProgramPathSettings} $PROGRAMDIRECTORY ProgramDirectory
	${ReadProgramSettings} $PROGRAMEXECUTABLE ProgramExecutable
	${ReadProgramPathSettings} $PROFILEDIRECTORY ProfileDirectory
	${ReadProgramPathSettings} $PLUGINSDIRECTORY PluginsDirectory
	${ReadProgramPathSettings} $TEMPDIR TempDirectory
	${ReadProgramPathSettings} $USERPROFILEDIRECTORY UserProfileDirectory
	${ReadProgramSettings} $DISABLESPLASHSCREEN DisableSplashScreen
	${ReadProgramSettings} $DISABLEWARNING DisableWarning
	${ReadProgramSettings} $SKIPCHROMEFIX SkipChromeFix
	${ReadProgramSettings} $SKIPCOMPREGFIX SkipCompregFix
	${ReadProgramSettings} $LOCALHOMEPAGE LocalHomePage
	${ReadProgramSettings} $REMOVEFLASH RemoveFlash

FunctionEnd

Function LoadDefaultSettings

	${DebugHeader} "NoINI"
	!insertmacro SplashSetText "Loading default settings..."

	StrCpy "$TORDIRECTORY" "$EXEDIR\App\${DEFAULTTORDIR}"
	StrCpy "$OutPath" "$EXEDIR\App"
	StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\App\Browser\${DEFAULTAPPDIR}"
	StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
	StrCpy "$PROFILEDIRECTORY" "$EXEDIR\Data\Browser\${DEFAULTPROFILEDIR}"
	StrCpy "$PLUGINSDIRECTORY" "$EXEDIR\Data\Browser\${DEFAULTPLUGINSDIR}"
	StrCpy "$TEMPDIR" "$EXEDIR\App\Browser\${DEFAULTTEMPDIR}"
	StrCpy "$USERPROFILEDIRECTORY" "$EXEDIR\Data\Browser\userprofile"
	StrCpy "$TIMERMAX" 180	
	StrCpy "$CONNECTIONMETHOD" ""
	StrCpy "$DISABLESPLASHSCREEN" "false"
	StrCpy "$DISABLEWARNING" "false"
	StrCpy "$SKIPCHROMEFIX" "false"
	StrCpy "$SKIPCOMPREGFIX" "false"
	StrCpy "$ISDEFAULTDIRECTORY" "true"
	StrCpy "$LOCALHOMEPAGE" ""
	StrCpy "$REMOVEFLASH" ""
	StrCpy "$WAITFORTORCIRCUIT" "true"
	StrCpy "$SUBMISSIVEMODE" "false"
	StrCpy "$DELETEDIRDATA" "false"
	StrCpy "$SOCKSPORT" "9050"
	StrCpy "$CONTROLPORT" "9051"
	StrCpy "$HTTPPROXYADDRESS" ""
	StrCpy "$HTTPPROXYPORT" ""
	StrCpy "$HTTPPROXYAUTH" ""
	StrCpy "$HTTPSPROXYADDRESS" ""
	StrCpy "$HTTPSPROXYPORT" ""
	StrCpy "$HTTPSPROXYAUTH" ""

FunctionEnd

Function LogINISettings

	${DebugHeader} "LogINISettings"

	${IF} $DEBUGON = 2
		${time::GetLocalTime} $R0
		FileWrite $DEBUGLOG "$R7 		TORDIRECTORY: $TORDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		OUTPATH: $OUTPATH$\r$\n"
		FileWrite $DEBUGLOG "$R7 		PROGRAMDIRECORY: $PROGRAMDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		PROGRAMEXECUTABLE: $PROGRAMEXECUTABLE$\r$\n"
		FileWrite $DEBUGLOG "$R7 		PROFILEDIRECTORY: $PROFILEDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		PLUGINSDIRECTORY: $PLUGINSDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		TEMPDIR: $TEMPDIR$\r$\n"
		FileWrite $DEBUGLOG "$R7 		USERPROFILEDIRECTORY: $USERPROFILEDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		TIMERMAX: $TIMERMAX$\r$\n"
		FileWrite $DEBUGLOG "$R7 		CONNECTIONMETHOD: $CONNECTIONMETHOD$\r$\n"
		FileWrite $DEBUGLOG "$R7 		DISABLESPLASHSCREEN: $DISABLESPLASHSCREEN$\r$\n"
		FileWrite $DEBUGLOG "$R7  		DISABLEWARNING: $DISABLEWARNING$\r$\n"
		FileWrite $DEBUGLOG "$R7 		SKIPCHROMEFIX: $SKIPCHROMEFIX$\r$\n"
		FileWrite $DEBUGLOG "$R7 		SKIPCOMPREGFIX: $SKIPCOMPREGFIX$\r$\n"
		FileWrite $DEBUGLOG "$R7 		ISDEFAULTDIRECTORY: $ISDEFAULTDIRECTORY$\r$\n"
		FileWrite $DEBUGLOG "$R7 		LOCALHOMEPAGE: $LOCALHOMEPAGE$\r$\n"
		FileWrite $DEBUGLOG "$R7 		WAITFORTORCIRCUIT: $WAITFORTORCIRCUIT$\r$\n"
		FileWrite $DEBUGLOG "$R7 		SUBMISSIVEMODE: $SUBMISSIVEMODE$\r$\n"
		FileWrite $DEBUGLOG "$R7  		DELETEDIRDATA: $DELETEDIRDATA$\r$\n"
		FileWrite $DEBUGLOG "$R7  		CONTROLPORT: $CONTROLPORT$\r$\n"
		FileWrite $DEBUGLOG "$R7  		SOCKSPORT: $SOCKSPORT$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPPROXYADDRESS: $HTTPPROXYADDRESS$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPPROXYPORT: $HTTPPROXYPORT$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPPROXYAUTH: $HTTPPROXYAUTH$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPSPROXYADDRESS: $HTTPSPROXYADDRESS$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPSPROXYPORT: $HTTPSPROXYPORT$\r$\n"
		FileWrite $DEBUGLOG "$R7  		HTTPSPROXYAUTH: $HTTPSPROXYAUTH$\r$\n"
		FileWrite $DEBUGLOG "$R7  		TORCOMMANDS: $TORCOMMANDS$\r$\n"
		FileWrite $DEBUGLOG "$R7  		DEBUGON: $DEBUGON$\r$\n"
	${EndIf}

FunctionEnd

#
# END
##########################

Function DetermineService

	${DebugHeader} "DetermineService"
	!insertmacro SplashSetText "Determining connection method..."
loop:

	${If} $CONNECTIONMETHOD = "" ; If there is no $CONNECTIONMETHOD set, autodetect for a .key file.	
		${DebugHeader} "DetermineService: $CONNECTIONMETHOD was unset, now checking for $EXEDIR\Data\XeroBank\XeroBank.key"
		!insertmacro SplashSetText "No connection method set"
		
		${DebugHeader} "DetermineService: Checking for clients and credentials..."

		${If} ${IsProcessRunning} "${VPNCLIENT}" ;If VPN Client is running
			${DebugHeader} "     ${VPNCLIENT} was found"
			!insertmacro SplashSetText "VPN running..."
			StrCpy $CONNECTIONMETHOD "2"
			;MessageBox MB_OK "${FullName} has detected you are running ${VPNCLIENT}. Please make sure the connection is active before continuing."
			Push "XeroBank"
			Return
		${ElseIf} ${IsProcessRunning} ${LEGACYVPN}
		${OrIf} ${IsProcessRunning} ${LEGACYVPNGUI}
			${DebugHeader} "     ${LEGACYVPN} but no ${VPNCLIENT} was found"
			!insertmacro SplashSetText "VPN running..."
			StrCpy $CONNECTIONMETHOD "2"
			;MessageBox MB_OK "${FullName} has detected you are running ${VPNCLIENT}. Please make sure the connection is active before continuing."
			Push "XeroBank"
			MessageBox MB_OK|MB_TOPMOST|MB_ICONEXCLAMATION "An alternate VPN client was found to be running. If it is not$\ra XeroBank connection this may affect your security and anonymity."
			Return
		${ElseIf} ${FileExists} "$EXEDIR\${VPNCLIENT}" 			;If the VPN client wasn't running, but was found
			!insertmacro SplashSetText "Initializing VPN..."
			Exec "$EXEDIR\${VPNCLIENT}"							;then start the VPN client 
			Sleep 12000 										;Wait 12 seconds for the VPN to connect
			StrCpy $CONNECTIONMETHOD "2"
			Push "XeroBank"
			Return
		${ElseIf} ${FileExists} "$EXEDIR\Data\XeroBank\XeroBank.key"
			${DebugHeader} "     XeroBank.key was found"
			!insertmacro SplashSetText "Found XeroBank credentials..."
			StrCpy $CONNECTIONMETHOD "1"
			Push "XeroBank"
			Return
		${Else}
			${DebugHeader} "     ${VPNCLIENT} was not found"
		${EndIf}

	${EndIf}

	${If} $CONNECTIONMETHOD = 0
		${DebugHeader} "DetermineService has determined we should start tor services"
		Push "Tor"
		Return
	${EndIf}

	${If} $CONNECTIONMETHOD > 0
		${DebugHeader} "DetermineService has determined we should start XeroBank services"
		Push "XeroBank"
		Return
	${EndIf}

	Goto loop ; XXX can this really happen?

FunctionEnd

Function SetEnvironmentVariables

	;set the plugins directory
	${DebugHeader} "PluginsEnvironment"

	${If} $PLUGINSDIRECTORY != ""
	${AndIf} ${FileExists} $PLUGINSDIRECTORY\*.*
		System::Call 'Kernel32::SetEnvironmentVariable(t "MOZ_PLUGIN_PATH", t "$PLUGINSDIRECTORY")'
	${EndIf}

	;set the profile directory
	${DebugHeader} "UserProfileEnvironment"

	${If} $USERPROFILEDIRECTORY != ""
	${AndIf} ${FileExists} $USERPROFILEDIRECTORY\*.*
		System::Call 'Kernel32::SetEnvironmentVariable(t "USERPROFILE", t "$USERPROFILEDIRECTORY")'
	${EndIf}

FunctionEnd


###############################
# Unload Functions

;Kill Browser
Function UnloadGUI ;Applies to xB VPN and keep running scenario...

	${DebugHeader} "UnloadGUI"
	!insertmacro SplashSetText "Unloading ${FULLNAME}..."
	${If} $SUBMISSIVEMODE != "true"
		${Unless} ${IsProcessRunning} "${MailClient}" ;If Torbird is running, don't shut down the connection client, just the Browser stuff
			Call KillConnectionClientProcess ;Kill the ConnectionClient
		${EndIf}
		Sleep 50
		KillProcDLL::KillProc "CONNECTIONCLIENT" ;If the conneciton client didn't exit, kill it outright.
	${EndIf}

	Call CleanUpProfilePrefs ;Save preference settings.
	Call ReplaceRegExtensions
	${DebugHeader} "UnloadGUI: Process Complete. ${NAME} exiting gracefully."
	
	FileClose $DEBUGLOG


	Quit

FunctionEnd

;Kill Browser and the GUIClient ie firefox, thunderbird, iceweasel, etc.
Function UnloadGUIandClient ;Applies to xB SSH and TOR

	${DebugHeader} "UnloadGUIandClient"

	${If} $SUBMISSIVEMODE != "true"

		;If Torbird is running, don't shut down the connection client, just the Browser stuff
		${Unless} ${IsProcessRunning} "${MailClient}"
			Call KillConnectionClientProcess ;Kill the ConnectionClient
		${EndIf}

		KillProcDLL::KillProc "PROGRAMEXECUTABLE"
		Sleep 50

		KillProcDLL::KillProc "$CONNECTIONCLIENT" ;If the conneciton client didn't exit, kill it outright.
		Sleep 50

	${EndIf}

	Call CleanUpProfilePrefs
	Call ReplaceRegExtensions
	Call RestoreJavaSettings

	${DebugHeader} "Unload${NAME}andClient: Process Complete. ${NAME} exiting gracefully."
	
	FileClose $DEBUGLOG
	Quit

FunctionEnd

/*

UNUSED

Function ForceUnloadGUIandClient

		${DebugHeader} "ForceUnloadGUIandClient"

		Sleep 200
		FileClose $DEBUGLOG

		Exec '"$EXEDIR\Data\Kill${NAME}.exe"'

		Quit

FunctionEnd
*/

# END
###############################

Function SetupProfileDirectory

	${DebugHeader} "SetupProfileDirectory"

	CreateDirectory $PROFILEDIRECTORY

	${Unless} ${DirectoryWritable} $PROFILEDIRECTORY
		CreateDirectory "$APPDATA\${NAME}\Profile"
		CopyFiles /SILENT $PROFILEDIRECTORY\*.* "$APPDATA\${NAME}\Profile"
		StrCpy $PROFILEDIRECTORY "$APPDATA\${NAME}\Profile"
	${EndIf}

	${Unless} ${DirectoryWritable} $PROFILEDIRECTORY
		;Write must have failed if we arrived here. Inform the user with a popup box.
		;XXX update message - should work for limited users now...
		${DebugHeader} "SetupProfileDirectory: no writable directory found"
		MessageBox MB_OK|MB_ICONINFORMATION|MB_TOPMOST "${FULLNAME} can not run from a read-only location and will now shut down.$\r$\rThis is usually caused by:$\r - Running from read-only media$\r - Limited user privileges$\r - Antivirus/IDS systems hindering ${FULLNAME}$\r - Out of disk space"
		Call UnloadGUI
	${EndIf}

	Delete "$PROFILEDIRECTORY\test.dat"

	${DebugHeader} "SetupProfileDirectory: $$PROFILEDIRECTORY is now $PROFILEDIRECETORY"

FunctionEnd

Function SetupWritableDirectories

	${DebugHeader} "SetupWritableDirectories"

	ClearErrors
	${Unless} ${DirectoryWritable} $TEMPDIR
		StrCpy $TEMPDIR "$TEMP\${NAME}"
	${EndIf}

	${DebugHeader} "SetupWritableDirectories: $$TEMPDIR is now $TEMPDIR"

FunctionEnd

Function CheckUpdate
	!insertmacro SplashSetText "Checking for updates..."
	${If} $CheckedUpdate == "1"
		Return
	${EndIf}

	Call CheckForUpdate

	StrCpy $CheckedUpdate "1"

FunctionEnd

;Splash screen easter egg
Function EasterEgg

	InitPluginsDir
	File /oname=$PLUGINSDIR\Browser.thx "xBBrowser.thx"
	newadvsplash::show /NOUNLOAD 23000 400 400 0x000000 /NOCANCEL $PLUGINSDIR\xBBrowser.thx
	Quit

FunctionEnd