// Firesomething extension for Mozilla Firefox
// Copyright (C) 2004-2007  Michael O'Rourke / Cosmic Cat Creations (http://www.cosmicat.com/)
// For licensing terms, please refer to readme.txt in this extension's XPInstall 
// package or its installation directory on your computer.

var FireSomething = {
	init: function() {
		this.Prefs = new FireSomething_PrefManager();
		this.DefaultName = document.getElementById("main-window").getAttribute("titlemodifier");
		this.initUserAgentStuff();
		var bSameName = this.getPref("extensions.firesomething.samename");
		if (bSameName) {
			var arrWin = this.getOtherWindows();
			if (arrWin.length > 0) {
				var oFS = arrWin[0].FireSomething;
				this.setNewBrowserName(oFS.Vendor, oFS.ShortName, oFS.TitleComment);
			} else {
				this.setNames();
			}
		} else {
			this.setNames();
		}
		window.addEventListener("focus", function() { FireSomething.setUserAgent(); }, false);
		this.Prefs.addPrefObserver("FireSomething.prefObserver");
	},
	init2: function() {
		document.getElementById("content").addEventListener("DOMTitleChanged", FireSomething.updateTitlebar, false);  // footnote "DOMTitle"
		this.updateTitlebar();
		this.DefaultAbout = document.getElementById("aboutName").getAttribute("label");
		setTimeout("FireSomething.setDelayedNames();", 100);
	},
	destruct: function() {
		try { this.Prefs.removePrefObserver(); } catch(ex) {}
	},
	getPref: function(strName) {
		return this.Prefs.getPref(strName);
	},
	prefObserver: function(subject, topic, prefName) {
		switch (prefName) {
			case "extensions.firesomething.images.about":
			case "extensions.firesomething.images.throbber1":
			case "extensions.firesomething.images.throbber2":
				return;
			default: this.setNames();
		}
	},
	setNames: function() {
		var bSameName = this.getPref("extensions.firesomething.samename");
		var bTopWin = (window == this.getWM().getMostRecentWindow(null));
		if (bSameName && !bTopWin) { return; }
		var vendor = this.getRandomName("extensions.firesomething.lists.vendors");
		var spacer = this.getRandomName("extensions.firesomething.lists.spacers");
		var prefix = this.getRandomName("extensions.firesomething.lists.prefixes");
		var suffix = this.getRandomName("extensions.firesomething.lists.names");
		var comment = this.getRandomName("extensions.firesomething.lists.comments");
		this.setNewBrowserName(vendor + spacer, prefix + suffix, comment);
		if (bSameName) { this.updateOtherWindows(); }
		if (bTopWin) { this.setUserAgent(); }
	},
	setNewBrowserName: function(strVendor, strShortName, strTitleComment) {
		this.Vendor = strVendor;
		this.ShortName = strShortName;
		this.TitleComment = strTitleComment;
		document.getElementById("main-window").setAttribute("titlemodifier", strVendor + strShortName + strTitleComment);
		this.updateTitlebar();
		setTimeout("FireSomething.setDelayedNames();", 100);
	},
	setDelayedNames: function() {
		try {
			this.updateTitlebar();
			var myRegExp = new RegExp(this.DefaultName, "");
			var strAbout = this.DefaultAbout.replace(myRegExp, this.Vendor + this.ShortName);
			document.getElementById("aboutName").setAttribute("label", strAbout);
		} catch(ex) { dump(ex + "\n"); }
	},
	updateTitlebar: function() {
		try { document.getElementById("content").updateTitlebar(); } catch(ex) {}
	},
	initUserAgentStuff: function() {
		var originalBrandVersion = this.Prefs.getService().getDefaultBranch(null)
				.getCharPref("general.useragent.extra.firefox");
		var myRegExp = new RegExp("\/", "");
		if (myRegExp.test(originalBrandVersion)) {
			this.OriginalBrand = RegExp.leftContext;
			this.OriginalVersion = RegExp.rightContext;
		}
	},
	setUserAgent: function() {
		var HTPH = Components.classes["@mozilla.org/network/protocol;1?name=http"]
				.getService(Components.interfaces.nsIHttpProtocolHandler);
		var userAgentTag = "";
		if (this.getPref("extensions.firesomething.useragent.rebrand")) {
			userAgentTag += this.ShortName + "/" + this.OriginalVersion;
			var comment = this.getPref("extensions.firesomething.useragent.comment");
			if (comment) {
				comment = comment.replace(/#name#/g, this.OriginalBrand);
				comment = comment.replace(/#version#/g, this.OriginalVersion);
				userAgentTag += " (" + comment + ")";
			}
			this.Prefs.setPref("general.useragent.extra.firefox", userAgentTag);
		} else {
			try { this.Prefs.getRootBranch().clearUserPref("general.useragent.extra.firefox"); } catch(ex) {}
		}
	},
	updateOtherWindows: function() {
		var arrWin = this.getOtherWindows();
		for (var i=0; i < arrWin.length; i++) {
			try { arrWin[i].FireSomething.setNewBrowserName(this.Vendor, this.ShortName, this.TitleComment); } catch(ex) { dump(ex + "\n"); }
		}
	},
	getOtherWindows: function() {
		var hWin, arrWin = new Array();
		var e = this.getWM().getEnumerator(null);
		while (e.hasMoreElements()) {
			hWin = e.getNext();
			if (hWin == window) { continue; }
			if (hWin.document.getElementById("main-window")) { arrWin[arrWin.length] = hWin; }
		}
		return arrWin;
	},
	getWM: function() {
		return Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
	},
	getRandomName: function(strPref) {
		var arrNames = this.getPref(strPref).split("|");
		return arrNames[this.getRandom(0, arrNames.length - 1)];
	},
	getRandom: function(min, max) {
		min = parseInt(min); max = parseInt(max);
		return Math.round((max - min) * Math.random()) + min;
	}
}

FireSomething.init();
window.addEventListener("load", function() { FireSomething.init2(); }, false);
window.addEventListener("unload", function() { FireSomething.destruct(); }, false);



/* *** FOOTNOTES ***

DOMTitle: In some cases, tabbrowser::updateTitlebar() will update the title correctly but then it
	will revert to using the previous 'titlemodifier' upon loading a new page. Forcing an update
	with the "DOMTitleChanged" event seems to rememdy this behavior with minimal overhead.
	
*/
